@@ -715,7 +715,12 @@ class QNexusWidget(qt.QWidget):
                         mcaList = NexusTools.getMcaList(h5file, entryName)
                         mcaShapeList = []
                         for mca in mcaList:
-                            dataset = h5file[mca]
+                            try:
+                                dataset = h5file[mca]
+                            except KeyError:
+                                dataset = h5file[NexusTools.sanitizeFilePath(h5file, 
+                                                                             mca,
+                                                                             entry=entryName)]
                             if hasattr(dataset, "shape"):
                                 mcaShapeList.append(dataset.shape)
                             else:
@@ -726,7 +731,10 @@ class QNexusWidget(qt.QWidget):
                             try:
                                 dataset = h5file[itemName]
                             except KeyError:
-                                dataset = h5file[NexusTools.sanitizeFilePath(h5file, itemName)]
+                                newItemName = NexusTools.sanitizeFilePath(h5file,
+                                                                         itemName,
+                                                                         entry=entryName)
+                                dataset = h5file[newItemName]
                             if hasattr(dataset, "shape"):
                                 scannedShapeList.append(dataset.shape)
                             else:
