@@ -1,8 +1,8 @@
 #/*##########################################################################
-# Copyright (C) 2004-2020 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2022 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
-# the ESRF by the Software group.
+# the ESRF.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
 # THE SOFTWARE.
 #
 #############################################################################*/
-__author__ = "V.A. Sole - ESRF Data Analysis"
+__author__ = "V.A. Sole - ESRF"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
@@ -32,6 +32,7 @@ import os
 import numpy
 import logging
 from PyMca5.PyMcaGraph.ctools import pnpoly
+from PyMca5.PyMcaGui.io import PyMcaFileDialogs
 from . import RGBCorrelatorGraph
 from . import ColormapDialog
 qt = RGBCorrelatorGraph.qt
@@ -1027,7 +1028,7 @@ class MaskImageWidget(qt.QWidget):
                                        self.__setBrush5)
             self.__brushMenu.addAction(QString("20 Image Pixel Width"),
                                        self.__setBrush6)
-        self.__brushMenu.exec_(self.cursor().pos())
+        self.__brushMenu.exec(self.cursor().pos())
 
     def __setBrush1(self):
         self.__brushWidth = 1
@@ -1098,7 +1099,7 @@ class MaskImageWidget(qt.QWidget):
     def _additionalSelectionMenuDialog(self):
         if self.__imageData is None:
             return
-        self._additionalSelectionMenu.exec_(self.cursor().pos())
+        self._additionalSelectionMenu.exec(self.cursor().pos())
 
     def _getSelectionMinMax(self):
         if self.colormap is None:
@@ -1745,7 +1746,7 @@ class MaskImageWidget(qt.QWidget):
         self.emitMaskImageSignal(ddict)
 
     def _saveToolButtonSignal(self):
-        self._saveMenu.exec_(self.cursor().pos())
+        self._saveMenu.exec(self.cursor().pos())
 
     def _saveMatplotlibImage(self):
         imageData = self.__imageData
@@ -1999,50 +2000,31 @@ class MaskImageWidget(qt.QWidget):
         if self.outputDir is not None:
             if os.path.exists(self.outputDir):
                 initdir = self.outputDir
-        filedialog = qt.QFileDialog(self)
-        filedialog.setFileMode(filedialog.AnyFile)
-        filedialog.setAcceptMode(qt.QFileDialog.AcceptSave)
-        filedialog.setWindowIcon(qt.QIcon(qt.QPixmap(IconDict["gioconda16"])))
         formatlist = ["TIFF Files *.tif",
                       "ASCII Files *.dat",
                       "EDF Files *.edf",
                       'CSV(, separated) Files *.csv',
                       'CSV(; separated) Files *.csv',
                       'CSV(tab separated) Files *.csv']
-        if hasattr(qt, "QStringList"):
-            strlist = qt.QStringList()
-        else:
-            strlist = []
-        for f in formatlist:
-                strlist.append(f)
-        if self._saveFilter is None:
+        if self._saveFilter in [None, ""]:
             self._saveFilter = formatlist[0]
-        if hasattr(filedialog, "setFilters"):
-            filedialog.setFilters(strlist)
-            filedialog.selectFilter(self._saveFilter)
-        else:
-            filedialog.setNameFilters(strlist)
-            filedialog.selectNameFilter(self._saveFilter)
-        filedialog.setDirectory(initdir)
-        ret = filedialog.exec_()
-        if not ret:
+        filelist, self._saveFilter = PyMcaFileDialogs.getFileList(parent=self,
+                        filetypelist=formatlist,
+                        message="Provide output file name", currentdir=initdir,
+                        mode="SAVE",
+                        getfilter=True,
+                        single=False, currentfilter=None, native=None)
+        if not len(filelist):
             return ""
-        filename = filedialog.selectedFiles()[0]
+        filename = filelist[0]
         if len(filename):
-            filename = qt.safe_str(filename)
             self.outputDir = os.path.dirname(filename)
-            if hasattr(filedialog, "selectedFilter"):
-                self._saveFilter = qt.safe_str(filedialog.selectedFilter())
-            else:
-                self._saveFilter = qt.safe_str(filedialog.selectedNameFilter())
             filterused = "." + self._saveFilter[-3:]
             PyMcaDirs.outputDir = os.path.dirname(filename)
             if len(filename) < 4:
                 filename = filename + filterused
             elif filename[-4:] != filterused :
                 filename = filename + filterused
-        else:
-            filename = ""
         return filename
 
     def saveImageList(self, filename=None, imagelist=None, labels=None):
@@ -2185,7 +2167,7 @@ def getImageMask(image, mask=None):
     Functional interface to interactively define a mask
     """
     w = MaskImageDialog(image=image, mask=mask)
-    ret = w.exec_()
+    ret = w.exec()
     if ret:
         mask = w.getMask()
     w = None
@@ -2256,7 +2238,7 @@ def test(filename=None, backend=None):
         print(ddict['event'])
 
     container.sigMaskImageWidgetSignal.connect(theSlot)
-    app.exec_()
+    app.exec()
     print(container.getSelectionMask())
 
 if __name__ == "__main__":
