@@ -1,8 +1,8 @@
 #/*##########################################################################
-# Copyright (C) 2004-2020 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2022 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
-# the ESRF by the Software group.
+# the ESRF.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
 # THE SOFTWARE.
 #
 #############################################################################*/
-__author__ = "V.A. Sole - ESRF Data Analysis"
+__author__ = "V.A. Sole - ESRF"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
@@ -56,6 +56,7 @@ except:
 
 from .PyMca_Icons import IconDict
 from PyMca5.PyMcaGui import PyMcaQt as qt
+from PyMca5.PyMcaGui.io import PyMcaFileDialogs
 
 if hasattr(qt, 'QString'):
     QString = qt.QString
@@ -145,7 +146,13 @@ class PlotWindow(PlotWidget.PlotWidget):
             self._xPos = qt.QLineEdit(self.graphBottom)
             self._xPos.setText('------')
             self._xPos.setReadOnly(1)
-            self._xPos.setFixedWidth(self._xPos.fontMetrics().width('##############'))
+            fm = self._xPos.fontMetrics()
+            fmtext = '##############'
+            if hasattr(fm, "maxWidth"):
+                self._xPos.setFixedWidth(fm.maxWidth() * len(fmtext))
+            else:
+                _logger.debug("Using obsolete method")
+                self._xPos.setFixedWidth(fm.width(fmtext))
             self.graphBottomLayout.addWidget(self._xPos)
 
             label=qt.QLabel(self.graphBottom)
@@ -155,7 +162,13 @@ class PlotWindow(PlotWidget.PlotWidget):
             self._yPos = qt.QLineEdit(self.graphBottom)
             self._yPos.setText('------')
             self._yPos.setReadOnly(1)
-            self._yPos.setFixedWidth(self._yPos.fontMetrics().width('##############'))
+            fm = self._yPos.fontMetrics()
+            fmtext = '##############'
+            if hasattr(fm, "maxWidth"):
+                self._yPos.setFixedWidth(fm.maxWidth() * len(fmtext))
+            else:
+                _logger.debug("Using obsolete method")
+                self._yPos.setFixedWidth(fm.width(fmtext))
             self.graphBottomLayout.addWidget(self._yPos)
             self.graphBottomLayout.addWidget(qt.HorizontalSpacer(self.graphBottom))
         widget.layout().addWidget(self.graphBottom, 0)
@@ -178,9 +191,9 @@ class PlotWindow(PlotWidget.PlotWidget):
                                        self.toggleCrosshairCursor)
             controlMenu.addAction(QString("Toggle Arrow Keys Panning"),
                                        self.toggleArrowKeysPanning)
-            controlMenu.exec_(self.cursor().pos())
+            controlMenu.exec(self.cursor().pos())
         else:
-            self._controlMenu.exec_(self.cursor().pos())
+            self._controlMenu.exec(self.cursor().pos())
 
     def setControlMenu(self, menu=None):
         self._controlMenu = menu
@@ -471,9 +484,9 @@ class PlotWindow(PlotWidget.PlotWidget):
                             self._getPrintConfigurationFromDialog)
             printMenu.addAction(QString("Print"),
                                        self.printGraph)
-            printMenu.exec_(self.cursor().pos())
+            printMenu.exec(self.cursor().pos())
         else:
-            self._printMenu.exec_(self.cursor().pos())
+            self._printMenu.exec(self.cursor().pos())
 
     def printGraph(self, *var, **kw):
         config = self.getPrintConfiguration()
@@ -507,7 +520,7 @@ class PlotWindow(PlotWidget.PlotWidget):
         oldConfig = self.getPrintConfiguration()
         self._printConfigurationDialog.setPrintConfiguration(oldConfig,
                                                     printer=self._printer)
-        if self._printConfigurationDialog.exec_():
+        if self._printConfigurationDialog.exec():
             self.setPrintConfiguration(\
                 self._printConfigurationDialog.getPrintConfiguration())
 
@@ -937,12 +950,16 @@ class PlotWindow(PlotWidget.PlotWidget):
                 if tip is not None:
                     action.setToolTip(tip)
                 menu.addAction(action)
+            elif self.pluginInstanceDict[m].__doc__:
+                action = qt.QAction(text, self)
+                action.setToolTip(self.pluginInstanceDict[m].__doc__)
+                menu.addAction(action)
             else:
                 menu.addAction(text)
             actionList.append(text)
             callableKeys.append(m)
         menu.hovered.connect(self._actionHovered)
-        a = menu.exec_(qt.QCursor.pos())
+        a = menu.exec(qt.QCursor.pos())
         if a is None:
             return None
         idx = actionList.index(a.text())
@@ -954,7 +971,7 @@ class PlotWindow(PlotWidget.PlotWidget):
                 msg.setWindowTitle("No plugins")
                 msg.setInformativeText(" Problem loading plugins ")
                 msg.setDetailedText(message)
-                msg.exec_()
+                msg.exec()
             return
         if idx == 1:
             dirName = qt.safe_str(qt.QFileDialog.getExistingDirectory(self,
@@ -994,7 +1011,7 @@ class PlotWindow(PlotWidget.PlotWidget):
                 actionList.append((text, pixmap, tip, action))
             #qt.QObject.connect(menu, qt.SIGNAL("hovered(QAction *)"), self._actionHovered)
             menu.hovered.connect(self._actionHovered)
-            a = menu.exec_(qt.QCursor.pos())
+            a = menu.exec(qt.QCursor.pos())
             if a is None:
                 return None
             idx = -1
@@ -1010,7 +1027,7 @@ class PlotWindow(PlotWidget.PlotWidget):
             msg.setText("An error has occured while executing the plugin:")
             msg.setInformativeText(str(sys.exc_info()[1]))
             msg.setDetailedText(traceback.format_exc())
-            msg.exec_()
+            msg.exec()
 
     def _actionHovered(self, action):
         tip = action.toolTip()
@@ -1349,7 +1366,7 @@ class PlotWindow(PlotWidget.PlotWidget):
             dialog = RenameCurveDialog.RenameCurveDialog(self,
                                                          oldLegend,
                                                          curveList)
-            ret = dialog.exec_()
+            ret = dialog.exec()
             if ret:
                 newLegend = dialog.getText()
                 self.renameCurve(oldLegend, newLegend, replot=True)
@@ -1497,7 +1514,7 @@ class PlotWindow(PlotWidget.PlotWidget):
                 msg.setIcon(qt.QMessageBox.Critical)
                 msg.setInformativeText(str(sys.exc_info()[1]))
                 msg.setDetailedText(traceback.format_exc())
-                msg.exec_()
+                msg.exec()
             return
         try:
             if sys.version_info.major >= 3:
@@ -1509,7 +1526,7 @@ class PlotWindow(PlotWidget.PlotWidget):
             msg.setIcon(qt.QMessageBox.Critical)
             msg.setInformativeText("Input Output Error: %s" % (sys.exc_info()[1]))
             msg.setDetailedText(traceback.format_exc())
-            msg.exec_()
+            msg.exec()
             return
         try:
             if not len(self._curveList):
@@ -1517,7 +1534,7 @@ class PlotWindow(PlotWidget.PlotWidget):
                 msg.setIcon(qt.QMessageBox.Critical)
                 msg.setInformativeText("No curve to be saved")
                 msg.setDetailedText(traceback.format_exc())
-                msg.exec_()
+                msg.exec()
                 return
             activeCurve = self.getActiveCurve()
             if activeCurve is None:
@@ -1591,12 +1608,9 @@ class PlotWindow(PlotWidget.PlotWidget):
             msg.setIcon(qt.QMessageBox.Critical)
             msg.setInformativeText("Error while saving: %s" % (sys.exc_info()[1]))
             msg.setDetailedText(traceback.format_exc())
-            msg.exec_()
+            msg.exec()
 
     def _getOutputFileName(self):
-        outfile = qt.QFileDialog(self)
-        outfile.setWindowTitle("Output File Selection")
-        outfile.setModal(1)
         filterlist = ['Specfile MultiScan *.dat',
                       'Specfile Scan *.dat',
                       'Specfile MCA  *.mca',
@@ -1607,25 +1621,20 @@ class PlotWindow(PlotWidget.PlotWidget):
                       'OMNIC CSV *.csv',
                       'Widget PNG *.png',
                       'Widget JPG *.jpg']
-        if hasattr(outfile, "setFilters"):
-            outfile.setFilters(filterlist)
-        else:
-            outfile.setNameFilters(filterlist)
-        outfile.setFileMode(outfile.AnyFile)
-        outfile.setAcceptMode(outfile.AcceptSave)
-        ret = outfile.exec_()
-        if not ret:
+        outputFile, outputFilter = PyMcaFileDialogs.getFileList(self,
+                    filetypelist=filterlist,
+                    message="Output File Selection",
+                    currentdir=None,
+                    mode="SAVE",
+                    getfilter=True,
+                    single=False,
+                    currentfilter=None, native=None)
+        if not len(outputFile):
             return None
-        if hasattr(outfile, "selectedFilter"):
-            outputFilter = qt.safe_str(outfile.selectedFilter())
-        else:
-            outputFilter = qt.safe_str(outfile.selectedNameFilter())
         filterused = outputFilter.split()
         filetype  = filterused[1]
         extension = filterused[2]
-        outputFile = qt.safe_str(outfile.selectedFiles()[0])
-        outfile.close()
-        del outfile
+        outputFile = outputFile[0]
         if len(outputFile) < 5:
             outputFile = outputFile + extension[-4:]
         elif outputFile[-4:] != extension[-4:]:
@@ -1682,5 +1691,5 @@ if __name__ == "__main__":
     #plot.removeCurve("dummy")
     #plot.addCurve(x, 2 * y, "dummy 2")
     #print("All curves = ",   plot.getAllCurves())
-    app.exec_()
+    app.exec()
     app = None
