@@ -52,6 +52,7 @@ from PyMca5.PyMcaIO import OmdaqLmf
 from PyMca5.PyMcaIO import JcampOpusStack
 from PyMca5.PyMcaIO import FsmMap
 from PyMca5.PyMcaIO import LabSpec6TxtMap
+from PyMca5.PyMcaIO import BrukerBCF
 from .QStack import QStack, QSpecFileStack
 try:
     from PyMca5.PyMcaGui.pymca import QHDF5Stack1D
@@ -202,17 +203,25 @@ class StackSelector(object):
                 # columns would be accepted as a Renishaw Map
                 # by other hand, I do not know how to handle
                 # that case as a stack.
+                _logger.info("RenishawMap")
                 stack = RenishawMap.RenishawMap(filelist[0])
                 omnicfile = True
             elif OmdaqLmf.isOmdaqLmf(filelist[0]):
+                _logger.info("OmdaqLmf")
                 stack = OmdaqLmf.OmdaqLmf(filelist[0])
                 omnicfile = True
             elif FsmMap.isFsmFile(filelist[0]):
+                _logger.info("FsmMap")
                 stack = FsmMap.FsmMap(filelist[0])
                 omnicfile = True
             elif JcampOpusStack.isJcampOpusStackFile(filelist[0]):
+                _logger.info("JcampOpusStack")
                 stack = JcampOpusStack.JcampOpusStack(filelist[0])
                 omnicfile = True
+            elif BrukerBCF.isBrukerBCFFile(filelist[0]):
+                _logger.info("Bruker bcf")
+                stack = BrukerBCF.BrukerBCF(filelist[0])
+                omnicfile = True
             else:
                 stack = QSpecFileStack()
 
@@ -241,16 +250,16 @@ class StackSelector(object):
                     if _logger.getEffectiveLevel() == logging.DEBUG:
                         raise
         if aifirafile:
-            masterStack = DataObject.DataObject()
-            masterStack.info = copy.deepcopy(stack.info)
-            masterStack.data = stack.data[:, :, 0:1024]
-            masterStack.info['Dim_2'] = int(masterStack.info['Dim_2'] / 2)
+            primaryStack = DataObject.DataObject()
+            primaryStack.info = copy.deepcopy(stack.info)
+            primaryStack.data = stack.data[:, :, 0:1024]
+            primaryStack.info['Dim_2'] = int(primaryStack.info['Dim_2'] / 2)
 
-            slaveStack = DataObject.DataObject()
-            slaveStack.info = copy.deepcopy(stack.info)
-            slaveStack.data = stack.data[:, :, 1024:]
-            slaveStack.info['Dim_2'] = int(slaveStack.info['Dim_2'] / 2)
-            return [masterStack, slaveStack]
+            secondaryStack = DataObject.DataObject()
+            secondaryStack.info = copy.deepcopy(stack.info)
+            secondaryStack.data = stack.data[:, :, 1024:]
+            secondaryStack.info['Dim_2'] = int(secondaryStack.info['Dim_2'] / 2)
+            return [primaryStack, secondaryStack]
         else:
             return stack
 
@@ -397,6 +406,9 @@ class StackSelector(object):
         if not HDF5:
             idx = fileTypeList.index("HDF5 Files (*.nxs *.hdf *.hdf5 *.h5)")
             del fileTypeList[idx]
+        if BrukerBCF.HAS_BCF_SUPPORT:
+            idx = fileTypeList.index("RTX Files (*.rtx *.RTX)")
+            fileTypeList.insert(idx+1, "BCF Files (*.bcf *.BCF)")
         message = "Open ONE indexed stack or SEVERAL files"
         return self._getFileList(fileTypeList, message=message,
                                  getfilter=getfilter)
@@ -501,10 +513,10 @@ if __name__ == "__main__":
     if type(stack) == type([]):
         #aifira like, two stacks
         widget.setStack(stack[0])
-        slave = QStackWidget.QStackWidget(master=False,
+        secondary = QStackWidget.QStackWidget(primary=False,
                                           rgbwidget=widget.rgbWidget)
-        slave.setStack(stack[1])
-        widget.setSlave(slave)
+        secondary.setStack(stack[1])
+        widget.setSecondary(secondary)
         stack = None
     else:
         widget.setStack(stack)
